

#ifndef __MENU_H__
#define __MENU_H__

#include "lcd.h"

// www.martinhubacek.cz
// Martin Hubek 21.4.2012

// Define some macros according to your display APIs,
// used languages and configure menu size

// Your code has to provide events to the variable keyPress
// you have to feed this values from your keyboard interrupt routine!
// Necessary values are BTN_LEFT, BTN_RIGHT, BTN_UP and BTN_DOWN

// If you use different languages, define them here
#define LANGUAGE_CZECH 0
#define LANGUAGE_ENGLISH 1

// Set rows/cols based on your font (for graphical displays)
#define ROW(x) ((x)*8)
#define COL(y) ((y)*6)
// For character LCDs use definitions below
//#define ROW(x) (x)
//#define COL(y) (y)

// Number of items on one screen
// Not including title
#define MENU_LINES 5

// Symbol which is displayed in front of the selected item
// This symbol doesn't appear when MENU_LINES == 1
#define ARROW_SYMBOL ">"
// How many spaces is between arrow symbol and menu item
// useful to set zero on smaller displays
#define ARROW_GAP 1

// Clear display
#define displayClear()	lcdBufferClear()

// Display string
#define displayString(str, posx, posy) lcdBufferString(str, posx, posy)
// If you have separate functions for set position and print text, use define below
//#define displayString(str, posx, posy) {lcdGotoXY(posx, posy); lcdBufferString(str);}

// Display number
#define displayNumber(str, posx, posy) lcdBufferNumber(str, posx, posy)
// If you have separate functions for set position and print number, use define below
//#define displayNumber(str, posx, posy) {lcdGotoXY(posx, posy); lcdBufferNumber(str)}

// Optional function to write buffer to display - comment if not used
#define displayDraw()		lcdBufferDraw()

// Optional edit dialogs
// Needs some more tweaking directly in menu.c according to your drawing routines
//#define MENU_EDIT_NUMBER
//#define MENU_EDIT_TIME
//#define MENU_EDIT_TEMPERATURE

// Some stdlib definitions according to your compiler
#define strlen(x) ustrlen(x)


// Nothing more to configure, that's all
// -----------------------------------------

#define int8_t int16_t
#define uint8_t unsigned char
#define int16_t short

extern char actualLanguage;

// Makro usnadujc zobrazen etzce podle navolenho jazyka
// #define lcd_putsLang(name) lcd_puts(name[actualLanguage])

int showMenu(char *array[]);
int8_t menuMsgBox(const char *str);

#ifdef MENU_EDIT_TEMPERATURE
uint8_t menuEditTemp(const char* str, int* temp, int increment);
#endif

#ifdef MENU_EDIT_TIME
int8_t menuEditTime(char *str, uint8_t* pHours, uint8_t* pMinutes);
#endif

#ifdef MENU_EDIT_NUMBER
uint8_t menuEditNumber(const char *str, int16_t *val, const char *unit, uint8_t inc, int16_t valMin, int16_t valMax);
#endif





extern volatile int menuPreselect;

#endif
